#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@File    : profile.py
@Time    : 2022/2/13 下午4:48
"""
version = "1.0.4"
jsonFilter = "version|cookie|dir|exit|format|info|json|query|debug|proxy|http|timeout|download|merge|multi|dir|capture|geometry|loop|player|fullscreen|start|end|length|no_audio|no_video|volume|init|language|itag|ccode|password|encoder|name"
